@@ -44,11 +44,12 @@ inline xt::xtensor<double, N + 2> Array<N>::I2() const
     #pragma omp parallel
     {
         auto unit = Cartesian3d::I2();
-        size_t stride = m_ndim * m_ndim;
+        auto shape = xt::xshape<m_ndim, m_ndim>();
+        size_t stride = m_stride_tensor2;
 
         #pragma omp for
         for (size_t i = 0; i < m_size; ++i) {
-            auto view = xt::adapt(&ret.data()[i * stride], xt::xshape<m_ndim, m_ndim>());
+            auto view = xt::adapt(&ret.data()[i * stride], shape);
             xt::noalias(view) = unit;
         }
     }
@@ -64,11 +65,12 @@ inline xt::xtensor<double, N + 4> Array<N>::II() const
     #pragma omp parallel
     {
         auto unit = Cartesian3d::II();
-        size_t stride = m_ndim * m_ndim * m_ndim * m_ndim;
+        auto shape = xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>();
+        size_t stride = m_stride_tensor4;
 
         #pragma omp for
         for (size_t i = 0; i < m_size; ++i) {
-            auto view = xt::adapt(&ret.data()[i * stride], xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>());
+            auto view = xt::adapt(&ret.data()[i * stride], shape);
             xt::noalias(view) = unit;
         }
     }
@@ -84,11 +86,12 @@ inline xt::xtensor<double, N + 4> Array<N>::I4() const
     #pragma omp parallel
     {
         auto unit = Cartesian3d::I4();
-        size_t stride = m_ndim * m_ndim * m_ndim * m_ndim;
+        auto shape = xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>();
+        size_t stride = m_stride_tensor4;
 
         #pragma omp for
         for (size_t i = 0; i < m_size; ++i) {
-            auto view = xt::adapt(&ret.data()[i * stride], xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>());
+            auto view = xt::adapt(&ret.data()[i * stride], shape);
             xt::noalias(view) = unit;
         }
     }
@@ -104,11 +107,12 @@ inline xt::xtensor<double, N + 4> Array<N>::I4rt() const
     #pragma omp parallel
     {
         auto unit = Cartesian3d::I4rt();
-        size_t stride = m_ndim * m_ndim * m_ndim * m_ndim;
+        auto shape = xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>();
+        size_t stride = m_stride_tensor4;
 
         #pragma omp for
         for (size_t i = 0; i < m_size; ++i) {
-            auto view = xt::adapt(&ret.data()[i * stride], xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>());
+            auto view = xt::adapt(&ret.data()[i * stride], shape);
             xt::noalias(view) = unit;
         }
     }
@@ -124,11 +128,12 @@ inline xt::xtensor<double, N + 4> Array<N>::I4s() const
     #pragma omp parallel
     {
         auto unit = Cartesian3d::I4s();
-        size_t stride = m_ndim * m_ndim * m_ndim * m_ndim;
+        auto shape = xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>();
+        size_t stride = m_stride_tensor4;
 
         #pragma omp for
         for (size_t i = 0; i < m_size; ++i) {
-            auto view = xt::adapt(&ret.data()[i * stride], xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>());
+            auto view = xt::adapt(&ret.data()[i * stride], shape);
             xt::noalias(view) = unit;
         }
     }
@@ -144,11 +149,12 @@ inline xt::xtensor<double, N + 4> Array<N>::I4d() const
     #pragma omp parallel
     {
         auto unit = Cartesian3d::I4d();
-        size_t stride = m_ndim * m_ndim * m_ndim * m_ndim;
+        auto shape = xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>();
+        size_t stride = m_stride_tensor4;
 
         #pragma omp for
         for (size_t i = 0; i < m_size; ++i) {
-            auto view = xt::adapt(&ret.data()[i * stride], xt::xshape<m_ndim, m_ndim, m_ndim, m_ndim>());
+            auto view = xt::adapt(&ret.data()[i * stride], shape);
             xt::noalias(view) = unit;
         }
     }
